/* -*- mode: c; c-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
undistort-opengl - Realtime pincushion distortion correction
Copyright 2008, Scott A. Kuhl
All rights reserved.

This file is part of undistort-opengl.

Undistort-opengl is free software: you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

Undistort-opengl is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
License for more details.

You should have received a copy of the GNU Lesser General Public License
along with undistort-opengl.  If not, see <http://www.gnu.org/licenses/>.
*/


uniform float k1;
uniform vec2 screenDimen;
uniform vec2 texScale;


#extension GL_ARB_texture_rectangle : enable
uniform sampler2DRect tex;

void main()
{

// Debug code: texture normally
//    gl_FragColor = texture2DRect(tex, gl_TexCoord[0].st);
//    return;

	// We are rendering onto a texture rectangle and texture
	// coordinates are [0,0] to (W,H) instead of [0,0] to (1,1).

	vec2 screenCenter = screenDimen/2.0;  /* Find the screen center */
	float norm = length(screenCenter);  /* Distance between corner and center */

	// get a vector from center to where we are at now (in screen space) and normalize it
	vec2 radial_vector = ( gl_TexCoord[0].st - screenCenter ) / norm;
	float radial_vector_len = length(radial_vector);
	vec2 radial_vector_unit = radial_vector / radial_vector_len;

/*
// Debug code:
gl_FragColor = vec4(ru, ru, ru, 1.0);
if(ru >= .95)
gl_FragColor = vec4(1.0, 0, 1.0, 1.0);
return;
*/

	// Compute the new distance from the screen center.
	float new_dist = radial_vector_len + k1 * pow(radial_vector_len,3.0);

	/* Now, compute texture coordinate we want to lookup. */

	// Find the coordinate we want to lookup
	vec2 warp_coord = radial_vector_unit * (new_dist * norm);

	// Scale the image vertically and horizontally to get it to fill the screen
	warp_coord = warp_coord * texScale;

	// Translate the coordinte such that the (0,0) is back at the screen center
	warp_coord = warp_coord + screenCenter;


	/* If we lookup a texture coordinate that is not on the texture, return a solid color */
	if ((warp_coord.s > float(screenDimen.x)  || warp_coord.s < 0.0) ||
	    (warp_coord.t > float(screenDimen.y) || warp_coord.t < 0.0))
		gl_FragColor = vec4(0,0,0,1); // black
	else
		gl_FragColor = texture2DRect(tex, warp_coord);  // lookup into the texture

}
